

function love.load()

	love.physics.setMeter(64) --Uses inbuilt physics to set length of world meter in pixels
	world = love.physics.newWorld(0, 9.81*64, true) --Creates the world to contain game objects

	--STARTING VALUES FOR CONSTANTS TO BE CALLED
	TimeConst = 1
	DebugState = 0
	
	objects = {} --Creates Table of Objects

	objects.ground = {} --Creates Table for Ground object properties
	objects.ground.body = love.physics.newBody(world, 1800/2, 1000) --Creates the Ground body to be in the 'world' world centred at X=1800/2, Y=1000, as a static object which cannot move
	objects.ground.shape = love.physics.newRectangleShape(1800, 64) --Creates the Ground shape of size X=1800, Y=64
	objects.ground.fixture = love.physics.newFixture(objects.ground.body, objects.ground.shape) --Fixes the ground body and shape together

	objects.rectangle = {}
	objects.rectangle.body = love.physics.newBody(world,900,500,"dynamic") --Creates the dynamic rectangle object centred at X=900, Y=500
	objects.rectangle.shape = love.physics.newRectangleShape(16, 32) --Creates the rectangle shape of size X=20, Y=50
	objects.rectangle.fixture = love.physics.newFixture(objects.rectangle.body, objects.rectangle.shape, 1)  --Fixes the rectangle body and shape together
	objects["rectangle"]["body"]:setFixedRotation(true) --FIXES THE ROTATION OF THE RECTANGLE OBJECT UPRIGHT
	

	map = {
		{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 },
		{ 1, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1 },
		{ 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1 },
		{ 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1 },
		{ 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 },
		{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0 }
	}

	
	for y=1, #map do
		for x=1, #map[y] do
			if map[y][x] == 1 then
				objects.square = {}
				objects.square.body = love.physics.newBody(world,(x * 64) - 32 , (y * 64) - 32) --Creates the square body objects to create the map
				objects.square.shape = love.physics.newRectangleShape(64, 64) --Creates the square shape of size X=64, Y=64
				objects.square.fixture = love.physics.newFixture(objects.square.body, objects.square.shape, 1)  --Fixes the square body and shape together
			end
		end
	end


	--STARTING CONSTANTS INVOLVING OBJECT VALUES

	X_Offset = (-objects.rectangle.body:getX() + 1000)
	Y_Offset = (-objects.rectangle.body:getY() + 500)

end


function love.update(dt)
	world:update(TimeConst*dt)

	X_Offset = (-objects.rectangle.body:getX() + 1000)
	Y_Offset = (-objects.rectangle.body:getY() + 500)

	--APPLY FORCE TO THE RECTANGLE OBJECT OR PAUSE THE GAME THROUGH USE OF DELTA TIME
	if love.keyboard.isDown("d") then
		objects.rectangle.body:applyForce(40, 0)
	end
	if love.keyboard.isDown("a") then
		objects.rectangle.body:applyForce(-40, 0)
	end
	if love.keyboard.isDown("w") then
		if	objects.rectangle.body:isTouching(objects.ground.body) == true or
			objects.rectangle.body:isTouching(objects.square.body) == true then
			objects.rectangle.body:applyForce(0, -800)
		end
	end
	if love.keyboard.isDown("space") then
		objects.rectangle.body:applyForce(0, -150)
	end

end


function love.draw()

	--BEGINNING OF CAMERA FOLLOW
	love.graphics.push()

	--HOW MUCH EVERYTHING IS TRANSLATED FOR CAMERA MOVEMENT
	love.graphics.translate(X_Offset, Y_Offset)

	--This one is really self explanatory
	love.graphics.setBackgroundColor(1, 1, 0.9)

	--SET COLOR FOR GROUND BODY OBJECT AND DRAW IT
	love.graphics.setColor(0.3, 0.6, 0.05)
	love.graphics.polygon("fill", objects.ground.body:getWorldPoints(objects.ground.shape:getPoints()))


	--DRAW MAP
	love.graphics.setColor(0.3, 0.8, 0.4)

	for y=1, #map do
		for x=1, #map[y] do
			if map[y][x] == 1 then
				love.graphics.rectangle("line", x * 64 - 64, y * 64 - 64, 64, 64)
			end
		end
	end


	--SET COLOR FOR RECTANGLE BODY OBJECT AND DRAW IT
	love.graphics.setColor(0.7, 0.4, 0.5)
	love.graphics.polygon("fill", objects.rectangle.body:getWorldPoints(objects.rectangle.shape:getPoints()))


	if DebugState == 1 then
	--SET COLOR FOR AND DRAW POSITION OF MOUSE POINTER WHICH IS FOLLOWING CAMERA (VERY IMPORTANT) (DARK GREEN)
	love.graphics.setColor(0.2, 0.3, 0.1)
	love.graphics.rectangle("fill", love.mouse.getX() - X_Offset, love.mouse.getY() - Y_Offset, 10, 10)
	else
	end

	love.graphics.pop()
	--END OF CAMERA FOLLOW

	if DebugState == 1 then
	--SET COLOR FOR AND DRAW CORRESPONDING POINTS FOR MOUSE CURSOR ON THE SCREEN THAT SHOULD BE, FOR RMB ROPEJOINT (GREEN) CORRECT PLACEMENT
	love.graphics.setColor(0, 0.7, 0.3)
	love.graphics.rectangle("fill", objects.rectangle.body:getX() + X_Offset, love.mouse.getY(), 10, 10)
	love.graphics.rectangle("fill", love.mouse.getX(), objects.rectangle.body:getY() + Y_Offset, 10, 10)
	else
	end

	--SET COLOR TO BLACK FOR TEXT
	love.graphics.setColor(0, 0, 0)

	if DebugState == 1 then
	--TEXT IN TOP LEFT OF SCREEN
	love.graphics.print("Mouse X: "..tostring(love.mouse.getX()), 10, 10)
	love.graphics.print("Mouse Y: "..tostring(love.mouse.getY()), 10, 30)
	love.graphics.print("Time Constant: "..tostring(TimeConst), 10, 50)
	love.graphics.print("Rectangle X: "..tostring(math.floor(objects.rectangle.body:getX())), 10, 110)
	love.graphics.print("X_Offset: "..tostring(math.floor(X_Offset)), 10, 130)

	love.graphics.print("Rectangle Y: "..tostring(math.floor(objects.rectangle.body:getY())), 200, 110)
	love.graphics.print("Y_Offset: "..tostring(math.floor(Y_Offset)), 200, 130)

	love.graphics.print("DebugState: "..tostring(DebugState), 400, 10)
	love.graphics.print("FPS: "..tostring(love.timer.getFPS( )), 400, 30)
	love.graphics.print("map[y][x]: "..tostring(map[y][x]), 400, 50)
	else
	end

end


function love.keyreleased(key, scancode)
	if key == "r" then
		if DebugState == 0 then
			DebugState = 1
		elseif DebugState == 1 then
			DebugState = 0
		end
	end
	if key == "escape" then
		love.event.quit()
	end
end

